# -*- coding: utf-8 -*-

from imio.smartweb.core.config import DIRECTORY_URL
from imio.smartweb.core.config import EVENTS_URL
from imio.smartweb.core.config import NEWS_URL
from imio.smartweb.core.contents import IDirectoryView
from imio.smartweb.core.contents import IEventsView
from imio.smartweb.core.contents import INewsView
from imio.smartweb.core.utils import get_json
from imio.smartweb.core.utils import get_wca_token
from imio.smartweb.core.utils import hash_md5
from plone import api
from plone.app.layout.viewlets.httpheaders import HeaderViewlet

import os


class OgpTagsViewlet(HeaderViewlet):
    _item = None
    _image = None
    _default_scale = "paysage_vignette"

    def update(self):
        if not self.request.form.get("u", None):
            # We are on the view
            self.set_ogp_informations_for_view()
        else:
            # We are on an item
            self.set_ogp_informations_for_item()
        super(OgpTagsViewlet, self).update()

    def set_ogp_informations_for_view(self):
        self._item = {
            "title": self.context.Title,
            "description": self.context.description,
            "url": self.context.absolute_url(),
        }
        dict_image = None
        image = self.context.image
        if image is not None:
            modified_hash = hash_md5(str(self.context.modified()))
            url = f"{self.context.absolute_url()}/@@images/image/paysage_vignette?cache_key={modified_hash}"
            dict_image = {
                "download": url,
                "filename": image.filename,
                "content-type": image.contentType,
                "scales": {
                    self._default_scale: {
                        "width": image.getImageSize()[0],
                        "height": image.getImageSize()[1],
                    }
                },
            }
        self._item["image"] = dict_image
        self._set_image()

    def set_ogp_informations_for_item(self):
        uid = self.request.form["u"]
        auth_source_url = ""
        endpoint = "@search"
        if IDirectoryView.providedBy(self.context):
            auth_source_url = DIRECTORY_URL
            client_id = os.environ.get("RESTAPI_DIRECTORY_CLIENT_ID")
            client_secret = os.environ.get("RESTAPI_DIRECTORY_CLIENT_SECRET")
        elif IEventsView.providedBy(self.context):
            auth_source_url = EVENTS_URL
            endpoint = "@events"
            client_id = os.environ.get("RESTAPI_EVENTS_CLIENT_ID")
            client_secret = os.environ.get("RESTAPI_EVENTS_CLIENT_SECRET")
        elif INewsView.providedBy(self.context):
            auth_source_url = NEWS_URL
            client_id = os.environ.get("RESTAPI_NEWS_CLIENT_ID")
            client_secret = os.environ.get("RESTAPI_NEWS_CLIENT_SECRET")
        auth = get_wca_token(client_id, client_secret)
        auth_source_url = (
            f"{auth_source_url}/{endpoint}?UID={uid}&fullobjects=1&include_items=true"
        )
        result_json = get_json(auth_source_url, auth=auth)
        if result_json:
            self._item = result_json["items"][0]
            self._set_image()

    def _set_image(self):
        if not self._item or not self._item.get("image", None):
            return None
        self._image = self._item.get("image")

    @property
    def image(self):
        return {} if not self._image else self._image

    @property
    def image_url(self):
        return self.scale(self._default_scale).get("download", "") or self.image.get(
            "download", ""
        )

    def scale(self, scale="paysage_vignette"):
        return self.image.get("scales", {}).get(scale, {})

    @property
    def title(self):
        if not self._item:
            return ""
        return self._item.get("title", "")

    @property
    def description(self):
        if not self._item:
            return ""
        return self._item.get("description", "")

    @property
    def url(self):
        if not self._item:
            return ""
        try:
            url = f'{self.request.ACTUAL_URL}?u={self.request.form["u"]}'
        except Exception:
            url = self.request.URL
        return url

    @property
    def site_name(self):
        return api.portal.get().title

    @property
    def image_type(self):
        return self.image.get("content-type", "")

    @property
    def image_width(self):
        return self.scale().get("width", "")

    @property
    def image_height(self):
        return self.scale().get("height")

    @property
    def image_alt(self):
        if not self.image:
            return ""
        return ""



import os

DIRECTORY_URL = os.environ.get("DIRECTORY_URL", "https://annuaire.enwallonie.be")
EVENTS_URL = os.environ.get("EVENTS_URL", "https://agenda.enwallonie.be")
NEWS_URL = os.environ.get("NEWS_URL", "https://actualites.enwallonie.be")
WCA_URL = os.environ.get(
    "WCA_URL", "https://agents.wallonie-connect.be/idp/oidc/token/"
)



# -*- coding: utf-8 -*-

from z3c.form.browser.radio import RadioWidget
from z3c.form.interfaces import IRadioWidget
from zope.interface import implementer
from zope.schema import TextLine
from zope.schema.interfaces import ITextLine


class ILinkField(ITextLine):
    """ """


class IIconsRadioWidget(IRadioWidget):
    """ """


@implementer(ILinkField)
class LinkField(TextLine):
    """ """


@implementer(IIconsRadioWidget)
class IconsRadioWidget(RadioWidget):
    """ """



# -*- coding: utf-8 -*-

from Acquisition import aq_parent
from imio.smartweb.core.contents.sections.base import ISection


def modified_section_content(obj, event):
    parent = aq_parent(obj)
    if parent is None:
        return
    if ISection.providedBy(parent):
        page = aq_parent(parent)
        page.reindexObject()



# -*- coding: utf-8 -*-

from imio.smartweb.common.faceted.utils import configure_faceted
from imio.smartweb.common.utils import is_log_active
from imio.smartweb.common.utils import remove_cropping
from imio.smartweb.core.behaviors.minisite import IImioSmartwebMinisite
from imio.smartweb.core.utils import safe_html
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.app.imagecropping import PAI_STORAGE_KEY
from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.namedfile.field import NamedBlobImage
from zope.annotation.interfaces import IAnnotations
from zope.component import getMultiAdapter
from zope.lifecycleevent import ObjectRemovedEvent
from zope.lifecycleevent.interfaces import IAttributes
from zope.globalrequest import getRequest
from zope.schema import getFields

import logging
import os

logger = logging.getLogger("imio.smartweb.core")


def moved_folder(obj, event):
    """We use IObjectMovedEvent instead of IObjectAddedEvent because Minisite interface is not yet
    provided when we use IObjectAddedEvent"""
    if not IImioSmartwebMinisite.providedBy(obj):
        return
    if type(event) is ObjectRemovedEvent:
        # We don't have anything to do if minisite is being removed
        return
    parent = event.newParent
    if INavigationRoot.providedBy(parent):
        # Minisites must always be under navigation root
        if not IImioSmartwebMinisite.providedBy(parent):
            # We are not in another minisite, everything is good
            return
    request = getRequest()
    minisite_settings = getMultiAdapter((obj, request), name="minisite_settings")
    minisite_settings.disable()
    api.portal.show_message(
        _(
            "Your Folder was a minisite but this behaviour has been disabled with this action"
        ),
        request,
        type="warning",
    )


def added_collection(obj, event):
    faceted_config_path = "{}/faceted/config/collection.xml".format(
        os.path.dirname(__file__)
    )
    configure_faceted(obj, faceted_config_path)


def added_sectionhtml(obj, event):
    obj.html = safe_html(obj.html)


def modified_sectionhtml(obj, event):
    obj.html = safe_html(obj.html)


def check_image_field(obj, interface, class_name, field_name):
    """Check if the field is an image field to see if we have cropping
    information to remove"""
    if field_name.startswith(f"{class_name}."):
        field_name = field_name.split(".")[-1]
    fields = getFields(interface)
    if field_name not in fields:
        # strange! the field is not found in schema class
        logger.warning(f"{field_name} not found in {interface.__identifier__}")
        return
    field = fields[field_name]
    if not isinstance(field, NamedBlobImage):
        # the field is not an image, nothing to do
        return
    annotations = IAnnotations(obj)
    scales = annotations.get(PAI_STORAGE_KEY)
    if scales is None:
        return
    scales_to_purge = []
    for scale in scales:
        if scale.startswith(field_name):
            scale_suffix = scale.split(f"{field_name}_")[-1]
            scales_to_purge.append(scale_suffix)
    if scales_to_purge:
        remove_cropping(obj, field_name, scales_to_purge)
        if is_log_active():
            logger.info(f"{scales_to_purge} croppings were PURGED for {field_name} !")
    elif is_log_active():
        logger.info(f"No existing cropping to purge for {field_name}.")


def modified_content(obj, event):
    if not hasattr(event, "descriptions") or not event.descriptions:
        return
    for d in event.descriptions:
        if not IAttributes.providedBy(d):
            # we do not have fields change description, but maybe a request
            continue

        class_name = d.interface.__identifier__.split(".")[-1]
        for field_name in d.attributes:
            check_image_field(obj, d.interface, class_name, field_name)



# -*- coding: utf-8 -*-

from imio.smartweb.core.viewlets.toolbar import AuthenticSourcesMenuItem
from imio.smartweb.core.viewlets.toolbar import SmartwebHelpMenuItem
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_INTEGRATION_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.tests.utils import get_json
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.memoize.ram import choose_cache
from zope.browsermenu.interfaces import IBrowserMenu
from zope.component import getUtility

import json
import os
import requests_mock


class TestToolbar(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.request = self.layer["request"]

    @requests_mock.Mocker()
    def test_sources_authentic_url(self, m):
        menu = getUtility(IBrowserMenu, name="plone_contentmenu", context=self.portal)
        m.get(
            "http://localhost:8080/Plone/@search?UID=7c69f9a738ec497c819725c55888ee32",
            text=json.dumps(get_json("resources/json_auth_sources_news_entity.json")),
        )
        m.get(
            "http://localhost:8080/Plone/@search?UID=7c69f9a738ec497c819725c55888ee31",
            text=json.dumps(get_json("resources/json_auth_sources_events_entity.json")),
        )
        m.get(
            "http://localhost:8080/Plone/@search?UID=396907b3b1b04a97896b12cc792c77f8",
            text=json.dumps(
                get_json("resources/json_auth_sources_directory_entity.json")
            ),
        )
        items = menu.getMenuItems(self.portal, self.request)
        authentic_sources_submenu = items[3].get("submenu")
        self.assertEqual(
            authentic_sources_submenu[0].get("title"),
            "localhost:8080/Plone/manage-news",
        )
        self.assertEqual(
            authentic_sources_submenu[1].get("title"),
            "localhost:8080/Plone/manage-events",
        )
        self.assertEqual(
            authentic_sources_submenu[2].get("title"),
            "localhost:8080/Plone/manage-directory",
        )

    def test_available_sources_authentic(self):
        menu = AuthenticSourcesMenuItem(self.portal, self.request)
        self.assertTrue(menu.available())
        logout()
        self.assertFalse(menu.available())

    def test_help_menu_url(self):
        menu = getUtility(IBrowserMenu, name="plone_contentmenu", context=self.portal)
        items = menu.getMenuItems(self.portal, self.request)
        smartweb_help_submenu = items[4].get("submenu")
        self.assertEqual(
            smartweb_help_submenu[0].get("action"),
            "https://docs.imio.be/iasmartweb/smartweb_v6",
        )
        self.assertEqual(
            smartweb_help_submenu[1].get("action"), "https://support.imio.be"
        )
        self.assertEqual(
            smartweb_help_submenu[2].get("action"),
            "https://my-formulaires.imio.be/ateliers-imio/ateliers-iasmartweb-1",
        )

        os.environ["help_menu_rtfm"] = "https://kamoulox.docs.be"
        cache = choose_cache("imio.smartweb.core.viewlets.toolbar.getMenuItems")
        cache.ramcache.invalidateAll()
        items = menu.getMenuItems(self.portal, self.request)
        smartweb_help_submenu = items[4].get("submenu")
        self.assertEqual(
            smartweb_help_submenu[0].get("action"), "https://kamoulox.docs.be"
        )

    def test_available_help_menu(self):
        menu = SmartwebHelpMenuItem(self.portal, self.request)
        self.assertTrue(menu.available())
        logout()
        self.assertFalse(menu.available())
